#vider tout
rm(list=ls())

#brancher les packages
library(xlsx) #lecture directe des fichiers EXCEL

#charger les donnes apprentissage
DTrain <- read.xlsx(file="pima-non-linearite.xlsx",header=T,sheetIndex=1)
str(DTrain)

#donnes test
DTest <- read.xlsx(file="pima-non-linearite.xlsx",header=T,sheetIndex=2)
str(DTest)

#construire le modle sans slection de variables
modele_glm <- glm(diabete ~ ., data = DTrain, family = binomial)
summary(modele_glm)

#prediction
pred1 <- factor(ifelse(predict(modele_glm,newdata=DTest,type="response")>0.5,"positive","negative"))

#librairie caret -- attention, il faut aussi installer e1071
library(caret)

#matrice de confusion + info. additionnelles
#attention, matrice de confusion transpose par rapport
# notre prsentation habituelle (en cours)
caret::confusionMatrix(data=pred1,reference=DTest$diabete,positive="positive")

#*****************************************************
#effectuer le test de Box-Tidwell pour chaque variable
#*****************************************************
for (j in 1:4){
  Z <- DTrain[,j] * log(DTrain[,j])
  print(paste("TEST pour ",toupper(colnames(DTrain)[j]),sep=""))
  modele.bt <- glm(diabete ~ ., data = cbind(DTrain,Z), family = binomial)
  print(summary(modele.bt))
}

#package rms
library(rms)

#rgression logistique
modele <- rms::lrm(diabete ~ ., data = DTrain, x = TRUE, y = TRUE)
print(modele)

#graphique des rsidus partiels
par(mfrow=c(2,2))
plot.lrm.partial(modele)

#*****************************
#transformations des variables
#inspires par le graphique
#*****************************
#prendre carr de diastolic
diastolic2 <- DTrain$diastolic^2
DTrainBis <- cbind(DTrain,diastolic2)

#prendre carr de serum
serum2 <- DTrain$serum^2
DTrainBis <- cbind(DTrainBis,serum2)

#discrtisation de triceps - codage non-embot
t1 <- ifelse(DTrain$triceps >= 20 & DTrain$triceps < 30,1,0)
DTrainBis <- cbind(DTrainBis,t1)

t2 <- ifelse(DTrain$triceps >= 30,1,0) 
DTrainBis <- cbind(DTrainBis,t2)

#nouvelle modlisation - modle avec variables transformes
modele.trans <- lrm(diabete ~ plasma+diastolic+diastolic2+triceps+t1+t2+serum+serum2,data=DTrainBis,x=T,y=T)
print(modele.trans)

#graphique des rsidus partiels
par(mfrow=c(4,2))
plot.lrm.partial(modele.trans)

#cration du modle au format glm(.)
modele.trans.glm <- glm(diabete ~ plasma+diastolic+diastolic2+triceps+t1+t2+serum+serum2,data=DTrainBis,family = binomial)
print(summary(modele.trans.glm))

#appliquer la transformation sur les donnes test
#prendre carr de diastolic
diastolic2 <- DTest$diastolic^2
DTestBis <- cbind(DTest,diastolic2)

#prendre carr de serum
serum2 <- DTest$serum^2
DTestBis <- cbind(DTestBis,serum2)

#discrtisation de triceps - codage non-embot
t1 <- ifelse(DTest$triceps >= 20 & DTest$triceps < 30,1,0)
DTestBis <- cbind(DTestBis,t1)

t2 <- ifelse(DTest$triceps >= 30,1,0) 
DTestBis <- cbind(DTestBis,t2)

#performances
pred2 <- factor(ifelse(predict(modele.trans.glm,newdata=DTestBis,type="response")>0.5,"positive","negative"))
caret::confusionMatrix(data=pred2,reference=DTest$diabete,positive="positive")

#************************************
#passer par la discrtisation
#pour tenir compte des non-linarits
#************************************

#package
library(discretization)

#discrtisation supervise
#guide par la variable cible diabete
#en dernire position dans notre base
#parce que nous sommes dans une analyse prdictive
disc <- discretization::mdlp(DTrain)
print(str(DTrain))
print(disc$cutp)

#nouveau data frame
print(head(disc$Disc.data))

#nouveau data frame pour l'apprentissage
DTrainDisc <- data.frame(lapply(disc$Disc.data[1:4],function(x){x-1}))

#retirer diastolic qui est une constante
DTrainDisc <- DTrainDisc[-2]

#ajouter la variable cible
DTrainDisc <- cbind(DTrainDisc,DTrain['diabete'])
str(DTrainDisc)

#refaire la rgression
modele.disc <- glm(diabete ~ .,data=DTrainDisc,family = binomial)
print(summary(modele.disc))

#coder les variables en test
#plasma
plasmaTest <- ifelse(DTest$plasma > disc$cutp[[1]],1,0)
print(plasmaTest)

#triceps
tricepsTest <- ifelse(DTest$triceps > disc$cutp[[3]], 1, 0)

#serum
#serumTest <- cut(DTest$serum,breaks=c(min(DTest$serum),disc$cutp[[4]],max(DTest$serum)),include.lowest = TRUE)
#serumTest <- as.numeric(unclass(serumTest)-1)
serumTest <- ifelse(DTest$serum > disc$cutp[[4]],1,0)
print(serumTest)

#nouveau data frame
DTestDisc <- data.frame(plasmaTest,tricepsTest,serumTest)
colnames(DTestDisc) <- c('plasma','triceps','serum')
str(DTestDisc)

#prdiction et valuation
pred3 <- factor(ifelse(predict(modele.disc,newdata=DTestDisc,type="response")>0.5,"positive","negative"))
caret::confusionMatrix(data=pred3,reference=DTest$diabete,positive="positive")
